package com.example.aped.communication;

import android.util.Log;

import com.android.volley.AuthFailureError;
import com.android.volley.NetworkResponse;
import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.toolbox.HttpHeaderParser;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Map;

/**
 * Custom {@link Request} to send xml files to the raspberry pi.
 */
public class FileOutputVolleyRequest extends Request<JSONObject> {

    /** Listener for response callback. **/
    private final Response.Listener<JSONObject> listener;

    /** File to be sent. **/
    private final File file;

    /** Headers of the response recieved from the raspberry. **/
    private Map<String, String> responseHeaders;

    /**
     * Constructor.
     * @param method {@link Request.Method}
     * @param url url the request will be sent to
     * @param pFile the file that will be sent
     * @param pListener a listener for the response callback
     * @param errorListener a listener for an error callback
     */
    public FileOutputVolleyRequest(final int method,
                               final String url,
                               final File pFile,
                               final Response.Listener<JSONObject> pListener,
                               final Response.ErrorListener errorListener) {
        super(method, url, errorListener);
        setShouldCache(false);
        this.listener = pListener;
        this.file = pFile;
    }

    /**
     * Overrides for the body content type.
     * @return xml body content type
     */
    @Override
    public String getBodyContentType() {
        return "application/xml; charset = utf-8";
    }

    /**
     * Defines the body content.
     * @return the body content
     * @throws AuthFailureError
     */
    @Override
    public byte[] getBody() throws AuthFailureError {
        try {
            BufferedReader bufferedReader = new BufferedReader(
                    new FileReader(file)
            );
            StringBuilder stringBuilder = new StringBuilder();
            String line;
            while ((line = bufferedReader.readLine()) != null) {
                stringBuilder.append(line);
                stringBuilder.append("\n");
            }
            bufferedReader.close();
            String requestBody = stringBuilder.toString();
            return requestBody.getBytes("utf-8");
        } catch (FileNotFoundException e) {
            Log.e("FileOutputVolleyRequest",
                    "could not find the output file: " + e.getMessage());
        } catch (IOException e) {
            Log.e("FileOutputVolleyRequest",
                    "error while reading the output file: "
                            + e.getMessage());
        }
        return null;
    }

    /**
     * Parses the response into an {@link JSONObject}.
     * @param response the response from the raspberry
     * @return a {@link Response<JSONObject>}
     */
    @Override
    protected Response<JSONObject> parseNetworkResponse(
            final NetworkResponse response) {
        responseHeaders = response.headers;

        JSONObject obj = null;
        try {
            obj = new JSONObject(new String(response.data));
        } catch (JSONException e) {
            Log.e("FileOutputVolleyRequest",
                    "Failed parsing JSON:" + e.getMessage());
        }
        return Response.success(
                obj, HttpHeaderParser.parseCacheHeaders(response)
        );
    }

    /**
     * Deliveres the Response to the response listener.
     * @param response the parsed response
     */
    @Override
    protected void deliverResponse(final JSONObject response) {
        listener.onResponse(response);
    }
}
